# Gui.py
# Name: Alex Cappiello
# AndrewID: acappiel
# Section: H
# Term project for 15-110, spring 2011
# All code original work unless otherwise noted
# icons NOT own work
#     http://www.iconarchive.com/show/farm-fresh-icons-by-fatcow.html and
#     http://www.iconarchive.com/show/isimple-system-icons-by-fasticon.html

from Tkinter import *
import FileIO, tkMessageBox, DrawingTools, tkColorChooser, Effects, Help

#quit message
def terminate(root):
    if ( root.canvas.data.unsavedChanges ):
        title = "Unsaved Changes"
        question = "You have unsaved changes. Would you like to save them?"
        response = tkMessageBox.askquestion( title, question,
                                             type = "yesnocancel" )
        if ( response == "yes" ):
            FileIO.save()
            root.quit()
        elif ( response == "no"):
            root.quit()
    else:
        root.quit()

###############################################################################
#fairly self documenting
#lambdas allow functions to be called with arguments

def fileMenu( menu, root ):
    fileMenu = Menu( menu, tearoff = False )
    menu.add_cascade( label = "File", menu = fileMenu )
    fileMenu.add_command( label = "New", command = FileIO.new )
    fileMenu.add_command( label = "Open...", command = FileIO.load )
    fileMenu.add_command( label = "Save", command = FileIO.save )
    fileMenu.add_command( label = "Save As...", command = FileIO.saveAs )
    fileMenu.add_separator()
    fileMenu.add_command( label = "Exit", command = lambda: terminate( root ) )

def imageMenu ( menu, root ):
    imageMenu = Menu( menu, tearoff = False )
    menu.add_cascade( label = "Image", menu = imageMenu )
    imageMenu.add_command( label = "Resize Canvas",
                           command = lambda: Effects.resizeCanvas( root ) )
    imageMenu.add_command( label = "Resize Image",
                           command = lambda: Effects.resizeImage( root ) )

def effectsMenu( menu, root ):
    effectsMenu = Menu( menu, tearoff = False )
    menu.add_cascade( label = "Effects", menu = effectsMenu )
    effectsMenu.add_command( label = "Grayscale",
                        command = lambda: DrawingTools.pixelMod( "grayscale" ) )
    effectsMenu.add_command( label = "Sepia",
                            command = lambda: DrawingTools.pixelMod( "sepia" ) )
    effectsMenu.add_command( label = "Brightness",
                       command = Effects.brightnessInit )
    effectsMenu.add_command( label = "Contrast",
                             command = lambda: Effects.contrast( root.canvas ) )
    effectsMenu.add_command( label = "Color",
                             command = lambda: Effects.color( root.canvas ) )
    effectsMenu.add_command( label = "Sharpness",
                             command = lambda: Effects.sharpness(root.canvas ) )
    
def helpMenu( menu, root ):
    helpMenu = Menu( menu, tearoff = False )
    menu.add_cascade( label = "Help", menu = helpMenu )
    helpMenu.add_command( label = "Function Overview",
                          command = lambda: Help.run( root ) )
    helpMenu.add_command( label = "About...", command = FileIO.about )

def createMenu( root ):
    menu = Menu( root )
    root.config( menu = menu )
    fileMenu( menu, root )
    imageMenu( menu, root )
    effectsMenu( menu, root )
    helpMenu( menu, root )

###############################################################################

def mainBar( root ):
    mainBar = Frame( root )
    mainBar.new = PhotoImage( file="gui/new.pgm" )
    new = Button( mainBar, image = mainBar.new, command = FileIO.new )
    new.pack( side=LEFT, padx = 2, pady = 2 )
    mainBar.load = PhotoImage( file = "gui/open.pgm" )
    load = Button( mainBar, image = mainBar.load, command = FileIO.load )
    load.pack( side=LEFT, padx = 2, pady = 2 )
    mainBar.save = PhotoImage( file = "gui/save.pgm" )
    save = Button( mainBar, image = mainBar.save, command = FileIO.save )
    save.pack( side = LEFT, padx = 2, pady = 2 )
    mainBar.saveAs = PhotoImage( file = "gui/saveAs.pgm" )
    saveAs = Button( mainBar, image = mainBar.saveAs, command = FileIO.saveAs )
    saveAs.pack( side = LEFT, padx = 2, pady = 2 )
    mainBar.close = PhotoImage( file = "gui/close.pgm" )
    close = Button( mainBar, image = mainBar.close,
                    command = lambda: terminate( root ) )
    close.pack( side = RIGHT, padx = 2, pady = 2 )
    mainBar.pack( side = TOP, fill = X )

def recordSize( sliderValue ):
    sliderValue = int( sliderValue )
    DrawingTools.setLineWidth( sliderValue )

def setColor( button, oldColor, identifier ):
    color = tkColorChooser.askcolor( initialcolor = oldColor )
    oldColor = color[ 1 ]
    if ( type( color[ 0 ]) != NoneType ):
        button.config( bg = oldColor )
        DrawingTools.setColor( color, identifier )

def basic( drawingBar ):
    drawingBar.eraserImg = PhotoImage( file="gui/eraser.pgm" )
    drawingBar.eraser = Button( drawingBar, image = drawingBar.eraserImg,
                            command = lambda: DrawingTools.setTool( "Eraser" ) )
    drawingBar.eraser.pack( side = TOP, padx = 2, pady = 2 )
    drawingBar.brushImg = PhotoImage( file="gui/brush.pgm" )
    drawingBar.brush = Button( drawingBar, image = drawingBar.brushImg,
                              command = lambda: DrawingTools.setTool( "Brush"))
    drawingBar.brush.pack( side = TOP, padx = 2, pady = 2 )
    drawingBar.eyedropImg = PhotoImage( file = "gui/eyedrop.pgm" )
    drawingBar.eyedrop = Button( drawingBar, image = drawingBar.eyedropImg,
                           command = lambda: DrawingTools.setTool( "Eyedrop" ) )
    drawingBar.eyedrop.pack( side = TOP, padx = 2, pady = 2 )
    drawingBar.fillImg = PhotoImage( file = "gui/fill.pgm" )
    drawingBar.fill = Button( drawingBar, image = drawingBar.fillImg,
                              command = lambda: DrawingTools.setTool( "Fill" ) )
    drawingBar.fill.pack( side = TOP, padx = 2, pady = 2 )

def shapes( drawingBar ):
    drawingBar.lineImg = PhotoImage( file = "gui/line.pgm" )
    drawingBar.line = Button( drawingBar, image = drawingBar.lineImg,
                              command = lambda: DrawingTools.setTool( "Line" ) )
    drawingBar.line.pack( side = TOP, padx = 2, pady = 2 )
    drawingBar.rectangleImg = PhotoImage( file = "gui/rectangle.pgm" )
    drawingBar.rectangle = Button( drawingBar, image = drawingBar.rectangleImg,
                         command = lambda: DrawingTools.setTool( "Rectangle" ) )
    drawingBar.rectangle.pack( side = TOP, padx = 2, pady = 2 )
    drawingBar.ellipseImg = PhotoImage( file = "gui/ellipse.pgm" )
    drawingBar.ellipse = Button( drawingBar, image = drawingBar.ellipseImg,
                           command = lambda: DrawingTools.setTool( "Ellipse" ) )
    drawingBar.ellipse.pack( side = TOP, padx = 2, pady = 2 )
    drawingBar.polygonImg = PhotoImage( file = "gui/polygon.pgm" )
    drawingBar.polygon = Button( drawingBar, image = drawingBar.polygonImg,
                           command = lambda: DrawingTools.setTool( "Polygon" ) )
    drawingBar.polygon.pack( side = TOP, padx = 2, pady = 2 )

def drawingBar( root ):
    global buttonColorA, buttonColorB, colorA, colorB
    buttonColorA = "#000000"
    buttonColorB = "#ffffff"
    main.drawingBar = Label( root )
    colorA = Button( main.drawingBar, bg = buttonColorA, width = 3, height = 2,
                     command = lambda: setColor( colorA, buttonColorA, 1 ) ) 
    colorA.pack(side = TOP, padx = 2, pady = 2 )
    colorB = Button( main.drawingBar, bg = buttonColorB, width = 3, height = 2,
                     command = lambda: setColor( colorB, buttonColorB, 2 ) )
    colorB.pack(side = TOP, padx = 2, pady = 2 )
    size = Scale( main.drawingBar, from_ = 0, to = 10,
                  command = recordSize)
    size.set( 2 )
    size.pack( padx = 2, pady = 2 )
    basic( main.drawingBar )
    shapes( main.drawingBar )
    main.drawingBar.pack( side = LEFT, fill = Y )

def createToolbar( root ):
    mainBar( root )
    drawingBar( root )

def createScrollbars( root ):
    global scrollX, scrollY
    scrollX = Scrollbar( root, orient = HORIZONTAL )
    scrollX.pack( side = BOTTOM, fill = X )
    scrollY = Scrollbar( root )
    scrollY.pack( side = RIGHT, fill = Y )

def createPointers():
    main.pointers = dict()
    main.pointers["Eraser"] = main.drawingBar.eraser
    main.pointers["Brush"] = main.drawingBar.brush
    main.pointers["Eyedrop"] = main.drawingBar.eyedrop
    main.pointers["Fill"] = main.drawingBar.fill
    main.pointers["Line"] = main.drawingBar.line
    main.pointers["Rectangle"] = main.drawingBar.rectangle
    main.pointers["Ellipse"] = main.drawingBar.ellipse
    main.pointers["Polygon"] = main.drawingBar.polygon
    

################################################################################

def configure( root ):
    global main
    class Struct: pass
    main = Struct()
    root.protocol( "WM_DELETE_WINDOW", lambda: terminate( root ) )
    createMenu( root )
    createToolbar( root )
    createScrollbars( root )
    createPointers()
